@@ -1,10 +1,12 @@
 using Gdk;
 using Gtk;
-using Gnome;
 using System.Collections;
+using System.Collections.Generic;
 using System;
 using FSpot;
+using FSpot.Utils;
 using FSpot.Xmp;
+using FSpot.UI.Dialog;
 using System.IO;
 using Mono.Unix;
 
@@ -16,14 +18,17 @@ public class ImportException : System.Exception {
 
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
+ 	RollStore rolls = FSpot.Core.Database.Rolls;
 	TagStore tag_store = FSpot.Core.Database.Tags;
 	bool recurse;
 	bool copy;
+	bool detect_duplicates;
 	string [] base_paths;
 	Tag [] tags;
 	Gtk.Window parent;
 
 	int count;
+	int duplicate_count;
 	XmpTagsImporter xmptags;
 
 	ArrayList import_info;
@@ -46,7 +51,7 @@ public class FileImportBackend : ImportBackend {
 		public ImportInfo (string original)
 		{
 			original_path = original;
-		        destination_path= null;
+		        destination_path = null;
 			Photo = null;
 		}
 	}
@@ -70,26 +75,17 @@ public class FileImportBackend : ImportBackend {
 
 	private void GetListing (System.IO.DirectoryInfo dirinfo, System.IO.FileInfo [] files, bool recurse)
 	{
-		System.Console.WriteLine ("Scanning {0}", dirinfo.FullName);
-		Hashtable exiting_entries = new Hashtable ();
+		Log.Debug ("Scanning {0} for new photos", dirinfo.FullName);
+		List<Uri> existing_entries = new List<Uri> ();
 
-		foreach (Photo p in store.Query (dirinfo)) {
-			foreach (uint id in p.VersionIds) {
-				string name;
-				if (id == Photo.OriginalVersionId)
-				        name = p.Name;
-				else 
-					name = (new System.IO.FileInfo (p.GetVersionPath (id))).Name;
+		foreach (Photo p in store.Query (new Uri (dirinfo.FullName + "/")))
+			foreach (uint id in p.VersionIds)
+				existing_entries.Add (p.VersionUri (id));
 
-				exiting_entries [name] = p;
-			}
-		}
-	
-		foreach (System.IO.FileInfo f in files) {
-			if (! exiting_entries.Contains (f.Name)) {
+		foreach (System.IO.FileInfo f in files)
+			if (! existing_entries.Contains (UriUtils.PathToFileUri (f.FullName)) && !f.Name.StartsWith (".")) {
 				AddPath (f.FullName);
 			}
-		}
 
 		if (recurse) {
 			foreach (System.IO.DirectoryInfo d in dirinfo.GetDirectories ()){
@@ -121,6 +117,9 @@ public class FileImportBackend : ImportBackend {
 		directories = new Stack ();
 		xmptags = new XmpTagsImporter (store, tag_store);
 
+		roll = rolls.Create ();
+		Photo.ResetMD5Cache ();
+
 		return import_info.Count;
 	}
 
@@ -149,9 +148,11 @@ public class FileImportBackend : ImportBackend {
 	public static string ChooseLocation (string path, Stack created_directories)
 	{
 		string name = System.IO.Path.GetFileName (path);
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		DateTime time = img.Date;
-		
+		DateTime time;
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (path)) {
+			time = img.Date;
+		}
+
 		string dest_dir = String.Format ("{0}{1}{2}{1}{3:D2}{1}{4:D2}",
 						 FSpot.Global.PhotoDirectory,
 						 System.IO.Path.DirectorySeparatorChar,
@@ -164,13 +165,13 @@ public class FileImportBackend : ImportBackend {
 			// Split dest_dir into constituent parts so we can clean up each individual directory in
 			// event of a cancel.
 			if (created_directories != null) {
-				string [] parts = dest_dir.Split (new char [] {'/'});
-				string nextPath = "";
+				string [] parts = dest_dir.Split (new char [] {System.IO.Path.DirectorySeparatorChar});
+				string nextPath = String.Empty;
 				for (int i = 0; i < parts.Length; i++) {
 					if (i == 0)
 						nextPath += parts [i];
 					else
-						nextPath += "/" + parts [i];
+						nextPath += System.IO.Path.DirectorySeparatorChar + parts [i];
 					if (nextPath.Length > 0) {
 						info = new System.IO.DirectoryInfo (nextPath);
 						// only add the directory path if it didn't already exist and we haven't already added it.
@@ -192,9 +193,11 @@ public class FileImportBackend : ImportBackend {
 		return dest;
 	}
 
-	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
+	public override bool Step (out StepStatusInfo status_info)
 	{
-		thumbnail = null;
+		Photo photo = null;
+		Pixbuf thumbnail = null;
+		bool is_duplicate = false;
 
 		if (import_info == null)
 			throw new ImportException ("Prepare() was not called");
@@ -210,43 +213,68 @@ public class FileImportBackend : ImportBackend {
 			string destination = info.OriginalPath;
 			if (copy)
 				destination = ChooseLocation (info.OriginalPath, directories);
-			
+
 			// Don't copy if we are already home
 			if (info.OriginalPath == destination) {
 				info.DestinationPath = destination;
-				photo = store.Create (info.DestinationPath, out thumbnail);
+
+				if (detect_duplicates)
+					photo = store.CheckForDuplicate (UriUtils.PathToFileUri (destination));
+
+				if (photo == null)
+					photo = store.Create (UriUtils.PathToFileUri (info.DestinationPath), roll.Id, out thumbnail);
+				else
+				 	is_duplicate = true;
 			} else {
 				System.IO.File.Copy (info.OriginalPath, destination);
 				info.DestinationPath = destination;
 
-				photo = store.Create (info.DestinationPath, info.OriginalPath, out thumbnail);
+				if (detect_duplicates)
+				 	photo = store.CheckForDuplicate (UriUtils.PathToFileUri (destination));
 
-				try {
-					File.SetAttributes (destination, File.GetAttributes (info.DestinationPath) & ~FileAttributes.ReadOnly);
-					DateTime create = File.GetCreationTime (info.OriginalPath);
-					File.SetCreationTime (info.DestinationPath, create);
-					DateTime mod = File.GetLastWriteTime (info.OriginalPath);
-					File.SetLastWriteTime (info.DestinationPath, mod);
-				} catch (IOException) {
-					// we don't want an exception here to be fatal.
+				if (photo == null)
+				{
+					photo = store.Create (UriUtils.PathToFileUri (info.DestinationPath),
+					                      UriUtils.PathToFileUri (info.OriginalPath),
+					                      roll.Id,
+					                      out thumbnail);
+				 	
+
+					try {
+						File.SetAttributes (destination, File.GetAttributes (info.DestinationPath) & ~FileAttributes.ReadOnly);
+						DateTime create = File.GetCreationTime (info.OriginalPath);
+						File.SetCreationTime (info.DestinationPath, create);
+						DateTime mod = File.GetLastWriteTime (info.OriginalPath);
+						File.SetLastWriteTime (info.DestinationPath, mod);
+					} catch (IOException) {
+						// we don't want an exception here to be fatal.
+					}
+				}
+				else
+				{
+					is_duplicate = true; 
+					System.IO.File.Delete (destination);
 				}
 			} 
 
-			if (tags != null) {
-				foreach (Tag t in tags) {
-					photo.AddTag (t);
+			if (!is_duplicate)
+			{
+				if (tags != null) {
+					foreach (Tag t in tags) {
+						photo.AddTag (t);
+					}
+					needs_commit = true;
 				}
-				needs_commit = true;
+
+				needs_commit |= xmptags.Import (photo, info.DestinationPath, info.OriginalPath);
+
+				if (needs_commit)
+					store.Commit(photo);
+
+				info.Photo = photo;
 			}
-
-			needs_commit |= xmptags.Import (photo, info.DestinationPath, info.OriginalPath);
-
-			if (needs_commit)
-				store.Commit(photo);
-			
-			info.Photo = photo;
 		} catch (System.Exception e) {
-			System.Console.WriteLine ("Error importing {0}\n{1}", info.OriginalPath, e.ToString ());
+			System.Console.WriteLine ("Error importing {0}{2}{1}", info.OriginalPath, e.ToString (), Environment.NewLine);
 			if (thumbnail != null)
 				thumbnail.Dispose ();
 
@@ -258,8 +286,8 @@ public class FileImportBackend : ImportBackend {
 									     Gtk.MessageType.Error,
 									     Gtk.ButtonsType.Cancel,
 									     Catalog.GetString ("Import error"),
-									     String.Format(Catalog.GetString ("Error importing {0}\n\n{1}"), info.OriginalPath, e.Message ));
-			errordialog.AddButton ("Skip", Gtk.ResponseType.Reject, false);
+									     String.Format(Catalog.GetString ("Error importing {0}{2}{2}{1}"), info.OriginalPath, e.Message, Environment.NewLine ));
+			errordialog.AddButton (Catalog.GetString ("Skip"), Gtk.ResponseType.Reject, false);
 			ResponseType response = (ResponseType) errordialog.Run ();
 			errordialog.Destroy ();
 			if (response == ResponseType.Cancel)
@@ -267,7 +295,11 @@ public class FileImportBackend : ImportBackend {
 		}
 
 		this.count ++;
-		count = this.count;
+
+		if (is_duplicate)
+		 	this.duplicate_count ++;
+
+		status_info = new StepStatusInfo (photo, thumbnail, this.count, is_duplicate);
 
 		return (!abort && count != import_info.Count);
 	}
@@ -307,6 +339,8 @@ public class FileImportBackend : ImportBackend {
 		}
 		// Clean up just created tags
 		xmptags.Cancel();
+
+		rolls.Remove (roll);
 	}
 
 	public override void Finish ()
@@ -321,17 +355,26 @@ public class FileImportBackend : ImportBackend {
 
 		import_info = null;
 		xmptags.Finish();
-		count = 0;
+		Photo.ResetMD5Cache ();
+
+		if (count == duplicate_count)
+		 	rolls.Remove (roll);
+
+		count = duplicate_count = 0;
+		//rolls.EndImport();    // Clean up the imported session.
 	}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse, Gtk.Window parent) : this (store, base_paths, false, recurse, null, parent) {}
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse, Gtk.Window parent) : this (store, base_paths, false, recurse, false, null, parent) {}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, Tag [] tags, Gtk.Window parent)
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, Tag [] tags, Gtk.Window parent) : this (store, base_paths, copy, recurse, false, null, parent) {}
+
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, bool detect_duplicates, Tag [] tags, Gtk.Window parent)
 	{
 		this.store = store;
 		this.copy = copy;
 		this.base_paths = base_paths;
 		this.recurse = recurse;
+		this.detect_duplicates = detect_duplicates;
 		this.tags = tags;
 		this.parent = parent;
 	}
@@ -350,7 +393,7 @@ public class FileImportBackend : ImportBackend {
 
 		Db db = new Db (path, true);
 
-		FileImportBackend import = new FileImportBackend (db.Photos, args [0],true);
+		FileImportBackend import = new FileImportBackend (db.Photos, args [0],true, this);
 
 		Console.WriteLine ("Preparing...");
 
